#  Copyright (C) 2007 by
#  Davide Lo Re <boyska@gmail.com>
#  Sergio Pistone <sergio_pistone@yahoo.com.ar>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the
#  Free Software Foundation, Inc.,
#  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

$LOAD_PATH << File.expand_path(File.dirname(__FILE__))

require 'utils/strings'
require 'lyrics'

require 'cgi'

class LyricsMania < Lyrics

	def LyricsMania.site_host()
		return 'www.lyricsmania.com'
	end

	def LyricsMania.site_name()
		return 'LyricsMania'
	end

	def LyricsMania.build_song_add_url( artist, title, album=nil, year=nil )
		return "http://#{site_host()}/add.html"
	end

	def LyricsMania.build_google_feeling_lucky_url( artist, title=nil )
		query =  Strings.google_search_quote( artist )
		query << ' ' << Strings.google_search_quote( title + ' lyrics' ) if ( title )
		return Strings.build_google_feeling_lucky_url( query, site_host() )
	end

	def build_lyrics_fetch_data( artist, title, album=nil, year=nil )
		artist = Strings.utf82latin1( artist )
		title  = Strings.utf82latin1( title )
		return { 'url' => build_google_feeling_lucky_url( artist, title ) }
	end

	def parse_lyrics( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", ' ' )

		lyrics_data = {}

		return lyrics_data if ( ! body.sub!( /^.* lyrics<\/h3>/, '' ) ) # metadata

		['Artist', 'Album'].each() do |key|
			if ( md =/#{key}: <b><a href=[^>]+>([^<]+)<\/a><\/b>/.match( body ) )
				lyrics_data[key.downcase()] = md[1].strip()
			end
		end
		['Year', 'Title'].each() do |key|
			if ( md =/#{key}: ([^<]+)<(br|\/td)>/.match( body ) )
				lyrics_data[key.downcase()] = md[1].strip()
			end
		end

		return lyrics_data if ( ! body.sub!( /^.*Correct<\/a> ?<\/td> ?<\/tr> ?<\/table> ?(<br>){0,10}/, '' ) )
		return lyrics_data if ( ! body.sub!( /(<b>)?<a href=.*$/, '' ) )

		body.sub!( /(<br ?\/>)?\[[^\]]+ Lyrics on http:\/\/www\.lyricsmania\.com\]/, '' )
		body.sub!( /^.*<\/a>/, '' ) # additional (optional) crap at the beginning
		body.gsub!( /<u>&lt;a[^<]+&lt;\/a&gt;<\/u>/, '' ) # yet more crap

		body.tr_s!( " \n\r\t", ' ' )
		body.gsub!( /\ ?<br ?\/?> ?/i, "\n" )
		body.strip!()

		lyrics_data['lyrics'] = body

		return lyrics_data

	end

	def build_suggestions_fetch_data( artist, title, album=nil, year=nil )
		return { 'url' => "http://#{site_host()}/search.php?k=#{CGI.escape(artist.gsub( /^\ *the\ */i, '' ))}&c=artist&submit=Search" }
		#return { 'url' => "http://#{site_host()}/search.php?k=#{CGI.escape(title.gsub( /^\ *the\ */i, '' ))}&c=title&submit=Search" }
	end

	# Returns the lyrics page body (nil if it couldn't be retrieved) and the response url
	def fetch_suggestions_page( url, post )

		body, url = super( url, post )

		# when the artist search returns more than one result we have to take the extra
		# step of searching the correct artist page amongst the returned results.
		if ( body != nil && body.sub!( /^.*artists found:<br><br>/m,'' ) )

			body = Strings.latin12utf8( body )

			log( ' - multiple artist pages found: parsing and scanning results... ', 0 ) if ( log?() )

			if ( ! body.sub!( /<td width=50% valign=top><br>.*$/m,'' ) || ! (md = /search.php\?k=([^&]+)&c/.match( url )) )
				log( 'ERROR parsing results' )
				return nil, nil
			else
				normalized_artist = Strings.normalize_token( CGI.unescape( md[1] ) )
			end

			url = nil
			body.split( /<h1>/mi ).each() do |entry|
				if ( (md = /<a href="(\/lyrics\/[^"]+)" title="([^"]+) lyrics">/.match( entry )) &&
					 normalized_artist == Strings.normalize_token( md[2] ) )
					url = "http://#{site_host()}#{md[1]}"
					log( 'ARTIST PAGE FOUND' ) if ( log?() )
					break
				end
			end

			if ( url == nil )
				log( 'NO ARTIST PAGE FOUND' ) if ( log?() )
				return nil, nil
			end

			log( " - fetching artist page... ", 0 ) if ( log?() )

			response, url = HTTP.fetch_page_get( url )
			body = response ? response.body() : nil

			if ( log?() )
				if ( body )
					log( 'OK' )
					log( body, 2 ) if ( long_log?() )
				else
					log( 'ERROR' )
				end
			end
		end

		return body, url

	end

	# returns an array of maps with following keys: url, artist, title
	def parse_suggestions( url, body, artist, title, album=nil, year=nil )

		body = Strings.latin12utf8( body )
		body.tr_s!( " \n\r\t", ' ' )

		suggestions = []

		if ( body.sub!( /^.*artists found:<br><br>/,'' ) )

			return suggestions if ( ! body.sub!( /<td width=50% valign=top><br>.*$/,'' ) )

			artist_lyrics = Strings.normalize_token( artist )
			artist_url = nil
			body.split( /<h1>/i ).each() do |entry|
				if ( md = /<a href="(\/lyrics\/[^"]+)" title="([^"]+) lyrics">/.match( entry ) )
					if ( artist_lyrics == Strings.normalize_token( md[2] ) )
						artist_url = "http://#{site_host()}#{md[1]}"
						break
					end
				end
			end

			return suggestions if ( artist_url == nil )

			response, url = HTTP.fetch_page_get( artist_url )
			return suggestions if ( response == nil || response.body() == nil )

			body = Strings.latin12utf8( response.body() )
			body.tr_s!( " \n\r\t", ' ' )

		end

		return suggestions if ( ! body.sub!( /^.*<\/iframe><br> ?<br> ?<table width=100%> ?<tr> ?<td width=50% valign=top>/,'' ) )
		return suggestions if ( ! body.sub!( /<TABLE cellSpacing=0 cellPadding=0 width="95%" border=0>.*$/,'' ) )

		body.split( /<a href=/ ).each() do |entry|
			if ( md = /"(\/lyrics\/[^"]+)" title="[^"]+"> ?([^>]+)<\/a><br>/.match( entry ) )
				suggestions << { 'url'=>"http://#{site_host()}#{md[1]}", 'artist'=>artist, 'title'=>md[2] }
			end
		end

		return suggestions

	end

end
